/*
 *
 *
 *   iobuf.h
 *
 *   Header for 'iobuffer' class, implementing nice I/O buffer (queue)
 *   for use between C++ application and C-style I/O functions.
 *
 *   Copyright (C) 2003, by the Grand River Dam Authority,
 *   All rights reserved.
 *
 *   Author: Matt Roberts
 *
 *
 */


#ifndef __SG_IOBUF
	#define __SG_IOBUF

	#define IOBUFFER_DEFAULT_LIMIT (4096)
	#define IOBUFFER_MINIMUM_LIMIT (1024)

	#include <deque>

	/*
	*
	*
	*   class iobuffer
	*
	*
	*/

	class iobuffer : public std::deque<unsigned char> {
		private:
			size_t io_limit;

		public:
			iobuffer(size_t limit);

			size_t push (const char * const cbuf);
			size_t push (const char * const cbuf, const size_t count);
			size_t pop (char *cbuf, const size_t count);

			bool full (void) { return (size() >= io_limit); };
	};


	inline iobuffer::iobuffer(size_t limit = IOBUFFER_DEFAULT_LIMIT) {
		/* if reasonable limit supplied, use it */
		if (limit >= IOBUFFER_MINIMUM_LIMIT)
			io_limit = limit;
		else
			io_limit = IOBUFFER_DEFAULT_LIMIT;
	}


	inline size_t iobuffer::push (const char * const cbuf) {
		insert(end(), cbuf, cbuf + strlen(cbuf));
		return strlen(cbuf);
	}


	inline size_t iobuffer::push (const char * const cbuf, const size_t count) {
		if (count == 0)
			return 0;
		insert(end(), cbuf, cbuf + count);
	}


	inline size_t iobuffer::pop (char *cbuf, const size_t limit) {
		if (empty())
			return 0;

		size_t count = limit;

		if (count > size())
			count = size();

		std::copy(begin(), begin() + count, cbuf);
		erase(begin(), begin() + count);

		return count;
	}

#endif /* __SG_IOBUF */
