/*
 *
 *
 *   printer.cc
 *
 *   Copyright (C) 2003, by the Grand River Dam Authority,
 *   All rights reserved.
 *
 *   Author: Matt Roberts
 *
 *
 */

#include "printer.h"

#include <unistd.h>
#include <fcntl.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/sysmacros.h>
#include <linux/lp.h>

#include <cctype>
#include <cstring>
#include <cerrno>

using namespace std;


Printer::Printer (const std::string &dev) : fd(-1) {
	open(dev);
}

Printer::Printer (void) : fd(-1) {
	/* noop */
}

void Printer::open (const std::string &dev) {
	// close, if already open
	Printer::close();

	// open the device
	int lp = ::open (dev.c_str(), O_WRONLY | O_NONBLOCK);

	// if we fail, throw an exception
	if (lp < 0)
		throw Printer::exception(this, errno);

	// Use the ioctl's to set the printer to DON'T CARE mode...
	::ioctl (lp, LPCAREFUL, 1);
	::ioctl (lp, LPABORT, 1);

	// assign the new FD
	fd = lp;
}

void Printer::close (void) throw() {
	// if the printer is open, close it
	if (fd > 0)
		::close(fd);
}

Printer::~Printer (void) throw() {
	Printer::close();
}

ssize_t Printer::print (const signed char *s, size_t len) {
	if (!s)
		return 0;
	return print(std::string(s, s+len));
}

ssize_t Printer::print (const unsigned char *s, size_t len) {
	if (!s)
		return 0;
	return print(std::string(s, s+len));
}

ssize_t Printer::print (const std::string &s) {
	if (!s.size())
		return 0;
	if (fd < 0)
		return 0;
	ssize_t rtn = ::write(fd, s.c_str(), s.size());
	if ((rtn < 0) && (errno != EAGAIN))
		throw Printer::exception(this, errno);
	return rtn;
}

// eof
