/*
 *
 *
 *   sgconfig.h
 *
 *   Copyright (C) 2003, by the Grand River Dam Authority,
 *   All rights reserved.
 *
 *   Author: Matt Roberts
 *
 *
 */


#ifndef __SGCONFIG_H
#define __SGCONFIG_H

	// my++ headers
	#include <my++/serialios.h>

	// C++ headers
	#include <list>
	#include <string>
	#include <exception>
	#include <iosfwd>


	/*
	 *
	 *   config_error
	 *
	 */

	class config_error : public std::exception {
		private:
			const std::string ce_what;
			const size_t ce_line;
		public:
			config_error(size_t l, const std::string &w)
				: ce_what(w), ce_line(l) { /* noop */ };
			const char *what (void) const throw() { return ce_what.c_str(); };
			size_t      line (void) const throw() { return ce_line;         };
			~config_error(void) throw() { /* noop */ };
	};


	/*
	 *
	 *  configuration
	 *
	 */
	
	struct port_config {
		// socket listen address
		std::string s_address;
		std::string s_owner;
		std::string s_group;
		mode_t s_mode;
	
		// tty configuration
		std::string s_device;
		my::serialios s_ios;

		// client limits
		size_t max_tx_queue;   // maximum characters in TX queue
		size_t client_limit;   // maximum connected clients
		time_t client_timeout; // max lifetime for dead client

		// set to TRUE to always read data, regardless of clients connected
		bool always_read;
	
		port_config(void);
		port_config(const port_config &c);
		~port_config(void) { /* noop */ };

		void clear (void);
		void dump (std::ostream &);
	};


	struct server_config {
		// location of config file
		std::string o_config;
		// location of PID file
		std::string o_pidfile;
		// set to daemonize
		bool o_daemon;
		// set to enable logging to stderr
		bool o_stderr;
		// set to enable logging to syslog
		bool o_syslog;

		// list of ports
		typedef std::list<port_config> pclist;
		pclist s_ports;

		// [cd]tors
		server_config(int argc, char **argv);
		server_config(const server_config &c);
		~server_config(void) { /* noop */ };

		// these are used by the ctor
		void add (const port_config &p) { s_ports.push_back(p); };
		void read_args (int argc, char **argv);
		void read_config ();
	};

#endif /* __SGCONFIG_H */
