/*
 *
 *
 *   termapp.h
 *
 *   Source code for TerminalApplication class common to the different
 *   SG-type terminals.
 *
 *   Copyright (C) 2003, by the Grand River Dam Authority,
 *   All rights reserved.
 *
 *   Author: Matt Roberts
 *
 *
 */


#ifndef __TERMAPP_H
#define __TERMAPP_H

#include <iostream>
#include <string>
#include <deque>

#include <my++/unixsocket.h>
#include <my++/tty.h>
#include <my++/select.h>

#include <my++/termios.h>
#include <my++/serialios.h>
#include <my++/conios.h>

#include <cstring>
#include <cstdio>

#include <time.h>
#include <curses.h>

#include "winbase.h"
#include "util.h"


/*
 *
 *
 *    class TerminalApplication
 *
 *
 */

class TerminalApplication {
	private: // types
		struct cursor_position {
			int x, y;
			cursor_position(void) { x = y = 0; };
			cursor_position(int h, int k) { x = h; y = k; };
			cursor_position(const cursor_position &cp) { x = cp.x; y = cp.y; };
		};

		class cursor_stack : public std::deque<cursor_position> {
			public:
				cursor_stack(void) { /* noop */ };
				void push(int x, int y) {
					push_front(cursor_position(x, y));
				};
				void pop(int &x, int &y) {
					if (size() <= 0) {
						x = y = (-1);
						return;
					}
					x = front().x;
					y = front().y;
					pop_front();
				}; 
		};

	private: // data
		std::string message;

		my::unixsocket sock;
		my::tty console;

		cursor_stack c_stack;
		WinBase &win_obj;

		WINDOW *termwin, *statwin;
		SCREEN *rootscr;

		int rows, cols;
		unsigned short rw_timeout;

	public:
		TerminalApplication (const std::string &tty_dev, WinBase &w_obj);
		~TerminalApplication (void);

		int run (const std::string &sock_addr);

	public:
		// use to set the r/w timeout
		void set_timeout(unsigned short seconds);

	private:  // utility functions used by run()
		void status_line (const std::string &message, const std::string &sock_path, bool ro);

		void saveCursor(void);
		void restoreCursor(void);

		void showCursor(void) { curs_set(1); };
		void hideCursor(void) { curs_set(0); };

		void reverse(void) { };
		void normal(void) { };
};


inline void TerminalApplication::set_timeout (unsigned short seconds) {
	if ((seconds == 0) || (seconds > 60))
		rw_timeout = seconds;
}

#endif /* __TERMAPP_H */
