/*
 *
 *   util.cc
 *
 *   Copyright (C) 2003, by the Grand River Dam Authority,
 *   All rights reserved.
 *
 *   Author: Matt Roberts
 *
 */

#include <my++/stype.h>
#include <string>

#include <stdio.h>
#include <ctype.h>

#include "util.h"

using namespace std;
using namespace my;


//
//
//   Home for SGD sockets
//
//

std::string sgsocket_default("/var/run/sgd");


//
//
//   getsockpath(path, default_dir)
//
//

std::string getsockpath (const std::string &path, const std::string &default_dir) {
	std::string result = rtrim(ltrim(path));

	if (result.empty())
		return result;

	if (path[0] == '/')
		return result;

	if (default_dir.empty())
		return result;

	std::string prefix = ltrim(rtrim(default_dir));
	
	if (prefix[prefix.size() - 1] != '/')
		prefix += '/';

	return std::string(prefix + result);
}

//
//
//   itoa() - turn a number into a string
//
//

string itoa (int arg) {
	static char data[16];
	snprintf(data, sizeof(data), "%d", arg);
	return string(data);
}


//
//
//   yes_or_no() - read boolean value
//
//

int yes_or_no (const string &value) {
	string newval = ltrim(rtrim(toLower(value)));
	if (
			(newval == "yes") || (newval == "true") ||
			(newval == "y") || (newval == "1")
	   ) return 1;
	if (
			(newval == "no") || (newval == "false") ||
			(newval == "n") || (newval == "0")
	   ) return 0;
	return -1;
}


//
//
//    get_keyword() - read the keyword from a config file line
//
//

string get_keyword (const string &line) {
	string::const_iterator si;

	for (si = line.begin(); si != line.end(); ++si) {
		if (isspace(*si))
			return string(line.begin(), si);
	}

	return line;
}


//
//
//    get_value() - read the value from a config file line
//
//

string get_value (const string &line) {
	string::const_iterator si;

	for (si = line.begin(); si != line.end(); ++si) {
		if (isspace(*si))
			return ltrim(string(si, line.end()));
	}

	return string();
}


//
//
//  gettime() - return a formatted time string
//
//

std::string gettime(void) {
	static char buf [128];
	static time_t now;
	static tm *now_tm;

	buf[0] = 0;
	now = time(NULL);
	now_tm = localtime(&now);
	strftime(buf, sizeof(buf), "%H:%M:%S", now_tm);

	return std::string(buf);
}


//
//
//   path_file (string)
//
//

std::string path_file (const std::string &s) {
	if (!s.size())
		return std::string();

	string::const_iterator si;
	string::const_iterator first;

	for (first = si = s.begin(); si != s.end(); ++si)
		if (*si == '/')
			first = si + 1;
	return std::string(first, s.end());
}


time_t now (void) {
	return time(NULL);
}


//
//
//   readline(fd, buf, len)
//
//

/*
bool readline (int fd, std::string &s) {
	char buf[128];
	ssize_t rtn;

	for (;;) {
		rtn = ::read(fd, buf, sizeof(buf));
		if (rtn == 0)
			return false;
		if (rtn < 0) {
			if (errno == EINTR)
				continue;
			else
				return false;
		}
	}
}
*/


//
//
//   readline(fd, buf, len)
//
//

/*
bool readline (int fd, const std::string &s) {
}
*/

// EOF
